# src/settings.py

import os

# lexi-system variables
COMPOSE_PROJECT_NAME = os.getenv("COMPOSE_PROJECT_NAME")

# Confluence environment variables
CONFLUENCE_PRIVATE_API_KEY = os.getenv("CONFLUENCE_PRIVATE_API_KEY")
CONFLUENCE_SPACE_URL = os.getenv("CONFLUENCE_SPACE_URL")
CONFLUENCE_SPACE_KEY = os.getenv("CONFLUENCE_SPACE_KEY")
CONFLUENCE_EMAIL_ADDRESS = os.getenv("CONFLUENCE_EMAIL_ADDRESS")
CONFLUENCE_SPACE_NAME = os.getenv("CONFLUENCE_SPACE_NAME")

ENVRC_TEMPLATE_FILE = os.getenv("ENVRC_TEMPLATE_FILE")
